function runSimulation(scene)
%%profile on;
clear global;

% Agents can access only some componets of the following
global SimOptions
global G
global botNos x xd xdCG xd_local
%       G is the global graph
%        .NodesN is the total number of agents -- all agents can access this
%        .A is the adjacency matrix -- Agent k can only check if it is connected to some other agent
%       x(k,:) is the position vector of agent k -- agent k can measure (x(l,:)-x(k,:)) for all neighbors l of k
%       xd(k,:) is the destination of agent k -- agent k can measure (xd(k,:)-x(k,:))

% All agents can access the following
global Agent_CommunicationInterval Agent_CommandInterval
global GlobalTimeStep
global CriticalRadius maxVelMagnitude
global SafetyFactor_CriticalRadius SafetyFactor_SubcriticalEdgeLength

% Agent k can access only Agent(k)
global Agent
% k-th element of the structure 'Agent' is accissible to the agent k
% Moreover agent k can access those other elements of structure 'Agent' to which they are connected
% Agent(k) has the following fields:
%         .t  -- the time shown on the clock maintained by agent k
%         .t_communication  -- Time stamp of last communication
%         .t_command        --Time stamp of last velocity command issue
%         .v  -- velocity of agent k [ .v is a 1x2 vector ]
%         .E  -- Information Age Matrix maintained by agent k [ .E is a G.NodesN x G.NodesN matrix ]
%         .A  -- Adjacency Matrix maintained by agent k [ .A is a G.NodesN x G.NodesN matrix ]
%         .R  -- Position vector matrix maintained by agent k

% Actually agent k should know Agent(k).v and (x(l,:)-x(k,:)) [for l \in N_k] only in its local coordinates
% However it doesnot matter as long as both Agent(k).v and (x(l,:)-x(k,:)) are measured by agent k in the same
%       coordinate frame. So without much loss of much genarality we choose the global frame for our convinience

% =================================================================

GlobalTimeStep = 1e-2;
GlobalTime = 0;
GlobalEndTime = Inf;
pauseTime = 0.0;

Agent_CommunicationInterval = 0.01;
Agent_CommandInterval = 0.05;

SafetyFactor_CriticalRadius = 1.0; %0.99;
SafetyFactor_SubcriticalEdgeLength = 1.0; %0.99;

SimOptions.DoVelocityDecoupling = true;

% =================================================================

switch scene
    case 'A'
        botNos = 6;
        xd = [-0.8 2.0; 0.8 2.0; -1.2 2.2; -1.2 1.8; 1.2 2.2; 1.2 1.8];
        x = [0.5 0.3; -0.4 0.2; -0.2 0.3; -0.1 -0.3; 0.2 -0.5; -0.4 0.1];
        CriticalRadius = 1.0;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;
    case 'B'
        botNos = 7;
        xd = [0.0 2.0; -1.0 2.0; 1.0 2.0; -1.2 2.2; -1.2 1.8; 1.2 2.2; 1.2 1.8];
        x = [-0.2 0.0; 0.5 0.3; -0.4 0.2; -0.2 0.3; -0.1 -0.3; 0.2 -0.5; -0.4 0.1];
        CriticalRadius = 0.8;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;
    case 'C1'
        botNos = 5;
        xd = [0.0 2.0; 0.5 2.5; -0.5 2.5; -0.5 1.5; 0.5 1.5];
        x = [0.0 0.0; 0.4 0.0; -0.4 0.0; -0.2 0.0; 0.2 0.0];
        CriticalRadius = 1.1;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;
    case 'C2'
        botNos = 5;
        xd = [0.0 2.0; 0.5 2.5; -0.5 2.5; -0.5 1.5; 0.5 1.5];
        x = [0.0 0.0; 0.4 0.0; -0.4 0.0; -0.2 0.0; 0.2 0.0];
        CriticalRadius = 0.8;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;
    case 'C3'
        botNos = 5;
        xd = [0.0 2.0; 0.5 2.5; -0.5 2.5; -0.5 1.5; 0.5 1.5];
        x = [0.0 0.0; 0.4 0.0; -0.4 0.0; -0.2 0.0; 0.2 0.0];
        CriticalRadius = 0.6;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;

    case 'D'
        botNos = 3;
        xd = [-0.4 2.5; 0.4 2.5; 0.0 2.5];
        x = [-0.4 0.0; 0.0 0.0; 0.4 0.0];
        CriticalRadius = 0.42;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;

    case 'X'
        botNos = 4;
        xd = [0.6 2.5; -0.6 2.5; -0.6 1.5; 0.6 1.5];
        x = 0.2*[0.0 0.0; 0.2 0.1; -0.2 0.0; -0.1 -0.1];
        CriticalRadius = 0.5;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;
    case 'Y'
        botNos = 3;
        xd = [1.0 1.5; -1.0 1.0; 1.0 0.5];
        x = [0.2 1.1; 0.0 1.0; 0.2 0.9];
        CriticalRadius = 1.0;
        maxVelMagnitude = 1.0;
        NumberOfSteps = 100;
        
    otherwise
        disp('Not a valid option!');
        return
end
% -----------------------------------------------

% % safety = 0.99; % Safety factor for regulating separation
% % 
% % updateStepsNo = botNos; % Nember of iterations required to propagate graph information
% % 
% % xOld = x;
% initiating bot positions - fully connected initial graph
%%for a=1:botNos,
    %%x(a,1) = rad*(2*rand - 1)/2;
    %%x(a,2) = rad*(2*rand - 1)/2;
%%end
%%x1Hist(:,1) = x(:,1);
%%x2Hist(:,1) = x(:,2);
% initiating destination of each bot
%%for a=1:botNos,
    %%xd(a,1) = 5*rad*(2*rand - 1);
    %%xd(a,2) = 5 + 2*rad*(2*rand - 1);
%%end

% Decoupling local graph paths and the global path
if SimOptions.DoVelocityDecoupling
    xdCG = mean(xd);
    xd_local = xd - repmat(xdCG, [botNos 1]);
else
    xd_local = xd;
end

% Generating the global graph
G = generateGraph(x, CriticalRadius);

fh = figure;
axes('DataAspectRatio',[1 1 1], 'PlotBoxAspectRatio',[1 1 1], 'XLim', [-2 2], 'YLim', [-1 3]); 
xlabel('X');
ylabel('Y');
titleString = sprintf('Number of agents: %d \n Radius over which an agent can communicate: %1.2f', botNos, CriticalRadius);
titleH = title(titleString, 'FontWeight', 'bold');
hold on;
grid off;
plot(xd(:,1), xd(:,2), 'k+');
edgeCount = 0;
for a=1:G.NodesN,
    for b=1:G.NodesN,
        if G.A(a,b) == 1
            hEdges(a+(b-1)*G.NodesN) = plot([x(a,1) x(b,1)], [x(a,2) x(b,2)], 'b-');
        else
            hEdges(a+(b-1)*G.NodesN) = plot([-100 -100], [-100 -100], 'b-');
        end
    end
end
hRobots = plot(x(:,1), x(:,2), 'ro');




% =================================================================


% Initiating the agents
for a=1:G.NodesN,
    Agent(a).t = 0;
    Agent(a).t_communication = -Inf;
    Agent(a).t_command = -Inf;
    Agent(a).v = [0 0];
    Agent(a).E(:,:) = Inf(G.NodesN, G.NodesN); % Information age matrix
    Agent(a).A(:,:) = zeros(G.NodesN, G.NodesN);
    Agent(a).R(:,:) = Inf(G.NodesN, G.NodesN);
end

% =================================================================

xOld = x;

% Beginning of loop
while GlobalTime < GlobalEndTime
    
    GlobalTime = GlobalTime + GlobalTimeStep;
    G = generateGraph(x, CriticalRadius);

    % Update each agent (random order of update to avoid bias)
    for k = randperm(G.NodesN)
        UpdateAnAgent(k);
    end

    % Integrate the position of each agent

% % % Eache agent communicate with its neighbors
% % % updating agents' adjuscency matrices for updateStepsNo times
% % for a=1:G.NodesN,
% %     for b=1:G.NodesN,
% %         
% %         % Updating the (a,b)th component of the adjuscency matrix of all the Agents for updateStepsNo iterations
% %         %for i=1:updateStepsNo,
% %             
% %             % Calculating the average of A(a,b) for p-th node over its neighbours
% %             for p=1:G.NodesN,
% %                 v(p) = Agent(p).A(a,b);
% %                 w(p) = Agent(p).R(a,b);
% %                 neighbourCount = 0;
% %                 for q=1:G.NodesN,
% %                     if G.A(p,q) == 1
% %                         v(p) = max([v(p) Agent(q).A(a,b)]);
% %                         w(p) = min([w(p) Agent(q).R(a,b)]);
% %                         neighbourCount = neighbourCount + 1;
% %                     end
% %                 end
% %                 %% v(p) = v(p) / (neighbourCount+1);
% %             end
% %             
% %             % Putting the newly calculated value into the actual variable A
% %             for p=1:G.NodesN,
% %                 %%Agent(p).AT(a,b) = sign(v(p) - Agent(p).A(a,b));
% %                 Agent(p).A(a,b) = v(p);
% %                 Agent(p).R(a,b) = w(p);
% %                 % p-th agent sets 1 and 0 for the agents with which it is connected
% %                 %%for q=1:G.NodesN,
% %                     %%if G.A(p,q) == 1
% %                         %%Agent(p).A(p,q) = 1;
% %                         %%Agent(p).A(q,p) = 1;
% %                     %%else
% %                         %%Agent(p).A(p,q) = 0;
% %                         %%Agent(p).A(q,p) = 0;
% %                     %%end
% %                 %%end
% %             end
% % 
% %         %end
% % 
% %     end
% % end

% Estimate the global adjuscency matrix from A and AT of each agent
%{
for p=1:G.NodesN,
    
    for a=1:G.NodesN,
        for b=1:G.NodesN,
            if a==p || b==p
                Agent(p).AE(a,b) = G.A(a,b);
            else
                Agent(p).AE(a,b) = Agent(p).AT(a,b);
            end
        end
    end
    
end
%}

% Find uncontrolled velocity for each agent
% Detect the critical edges for each agent
% modify velocity accordingly
% % for a=1:G.NodesN,
% %     
% %      % Uncontrolled velocity calculation
% %     Agent(a).v = 5 * (xd(a,:) - x(a,:));
% %     Magni = norm(Agent(a).v);
% %     if Magni > maxVelMagnitude
% %         Agent(a).v = maxVelMagnitude * Agent(a).v / Magni;
% %     end
% %     
% %     % Detect critical neighbours
% %     % critNeigh = detectCriticalEdges(Agent(a).A, a);
% %     critNeigh = detectPotentiallyCriticalEdges(Agent(a).A, Agent(a).R, a);
% %     
% %     % modify the velocities
% %     scaleFactor = 1;
% %     scaleMat = eye(2);
% %     Vel = (Agent(a).v)';
% %     for b=critNeigh,
% %         thisScaleMat = modifyVelocity_1(Vel, (x(b,:)-x(a,:))', safety*rad, dt);
% %         scaleMat = thisScaleMat * scaleMat;
% %     end
% %     newVel = scaleMat * Vel;
% %     for b=critNeigh,
% %         scaleFactor(b) = modifyVelocity_2(newVel, (x(b,:)-x(a,:))', safety*rad, dt);
% %     end
% %     %b,
% %     %min(scaleFactor),
% %     Agent(a).v = (min(scaleFactor)* newVel)'; %????
% % end

% Integrating
for a=1:G.NodesN,
    x(a,:) = x(a,:) + GlobalTimeStep*Agent(a).v;
end


% Plotting graph
%%fh = figure(fh);
%%clf;
%%axes('DataAspectRatio',[1 1 1], 'PlotBoxAspectRatio',[1 1 1], 'XLim', [-2 2], 'YLim', [-1 3]); 
%%xlabel('X');
%%ylabel('Y');
%%hold on;
%%grid off;
for a=1:G.NodesN,
    plot([xOld(a,1) x(a,1)], [xOld(a,2) x(a,2)], 'm-');
end
xOld = x;
for a=1:G.NodesN,
    for b=1:G.NodesN,
        if G.A(a,b) == 1
            set(hEdges(a+(b-1)*G.NodesN),'XData',[x(a,1) x(b,1)],'YData',[x(a,2) x(b,2)]);
        else
            set(hEdges(a+(b-1)*G.NodesN),'XData',[],'YData',[]);
        end
    end
end
set(hRobots,'XData',x(:,1),'YData',x(:,2));
set(titleH, 'String', [titleString,sprintf('\n t = %f', GlobalTime)]);
drawnow;
%~M(StepCount) = getframe;


% End of loop
pause(pauseTime);
end

%%profile viewer;
%%profile off;


% save('Connect6_1.mat', 'M', 'xd', 'x1Hist', 'x2Hist');
%~movie(M,1);
%~movie2avi(M,[scene '_Connectivity.avi']);

    
%{
G.A,
Agent(2).A,
Agent(2).AT,
Agent(2).AE,
detectCriticalEdges(Agent(2).AE, 2),
x(2,:),
%}




% ======================================================================
